"""
agipy enables you to destroy cloud infrastructure in a hurry.
Copyright (C) 2019  Nils Müller<shimst3r@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>
"""

import os
from collections import namedtuple
from unittest.mock import patch

import click.testing

from . import azure

# Some of the test cases use patching for mocking third party dependencies.
# Because the unittest.mock.patch function expects those patched objects to
# be present as (then unused) function arguments, this pylint warning will
# be disabled.
# pylint: disable=unused-argument

EnvironmentVariable = namedtuple("EnvironmentVariable", ["env", "arg", "value"])

ENV_VARS = [
    EnvironmentVariable("AZURE_CLIENT_ID", "--client-id", "Azure_Client_Id"),
    EnvironmentVariable(
        "AZURE_CLIENT_SECRET", "--client-secret", "Azure_Client_Secret"
    ),
    EnvironmentVariable(
        "AZURE_SUBSCRIPTION_ID", "--subscription-id", "Azure_Subscription_Id"
    ),
    EnvironmentVariable("AZURE_TENANT_ID", "--tenant-id", "Azure_Tenant_Id"),
]


def test_azure_without_prefix():
    """
    This test case verifies that the azure provider will not run when no
    prefix argument is provided.
    """
    expected_exit_code = 2
    expected_output = (
        "Usage: azure [OPTIONS]\n"
        'Try "azure --help" for help.\n'
        "\n"
        'Error: Missing option "--prefix".\n'
    )

    args = ["--yes"]

    runner = click.testing.CliRunner()
    actual = runner.invoke(azure.azure, args=args)

    assert actual.exit_code == expected_exit_code
    assert actual.output == expected_output


def test_azure_without_environment_variables():
    """
    This test case verifies that the azure provider will not run when the
    necessary environment variables are not set.
    """
    expected_exit_code = 1
    expected_output = "azure provider is missing the 'AZURE_CLIENT_ID'.\n"

    args = ["--prefix=test", "--yes"]

    runner = click.testing.CliRunner()
    actual = runner.invoke(azure.azure, args=args)

    assert actual.exit_code == expected_exit_code
    assert actual.output == expected_output


@patch("azure.common.credentials.ServicePrincipalCredentials")
@patch("azure.mgmt.resource.ResourceManagementClient")
def test_azure_with_prefix_and_environment_variables(cred, client):
    """
    This test case verifies that the azure provider will run properly if all
    necessary environment variables are set.

    In order to avoid calls to the Microsoft Azure API, both external
    dependencies are patched as mock objects.
    """
    expected_exit_code = 0
    expected_output = f"Found no deletable resource groups for prefix test.\n"

    args = ["--prefix=test", "--yes"]

    with patch.dict(os.environ, {env.env: env.value for env in ENV_VARS}):
        runner = click.testing.CliRunner()
        actual = runner.invoke(azure.azure, args=args)

        assert actual.exit_code == expected_exit_code
        assert actual.output == expected_output


def test_azure_with_permutations_of_missing_environment_variables(subtests):
    """
    This test case verifies that the azure provider will not run when at least
    one of the necessary environment variables is not set if called without
    additional command line arguments.
    """
    for env_var in ENV_VARS:
        expected_exit_code = 1
        expected_output = f"azure provider is missing the '{env_var.env}'.\n"

        args = ["--prefix=test", "--yes"]
        subset = {var.env: var.value for var in ENV_VARS if var != env_var}

        with patch.dict(os.environ, subset):
            runner = click.testing.CliRunner()
            actual = runner.invoke(azure.azure, args=args)

            with subtests.test(msg=f"Testing without {env_var} being set"):
                assert actual.exit_code == expected_exit_code
                assert actual.output == expected_output


@patch("azure.common.credentials.ServicePrincipalCredentials")
@patch("azure.mgmt.resource.ResourceManagementClient")
def test_azure_with_prefix_and_command_line_arguments(cred, client):
    """
    This test case verifies that the azure provider will run properly if all
    necessary command line arguments are provided.

    In order to avoid calls to the Microsoft Azure API, both external
    dependencies are patched as mock objects.
    """
    expected_exit_code = 0
    expected_output = f"Found no deletable resource groups for prefix test.\n"

    args = (
        ["--prefix=test"] + [f"{env.arg}={env.value}" for env in ENV_VARS] + ["--yes"]
    )

    with patch.dict(os.environ, {}):
        runner = click.testing.CliRunner()
        actual = runner.invoke(azure.azure, args=args)

        assert actual.exit_code == expected_exit_code
        assert actual.output == expected_output


def test_azure_without_env_vars_and_permutations_of_missing_arguments(subtests):
    """
    This test case verifies that the azure provider will not run when the
    necessary environment variables are not set and at least one of the
    necessary command line arguments is not provided.
    """
    for env_var in ENV_VARS:
        with patch.dict(os.environ, {}):
            expected_exit_code = 1
            expected_output = f"azure provider is missing the '{env_var.env}'.\n"

            subset = {var.arg: var.value for var in ENV_VARS if var != env_var}
            args = (
                ["--prefix=test"] + [f"{k}={v}" for k, v in subset.items()] + ["--yes"]
            )

            runner = click.testing.CliRunner()
            actual = runner.invoke(azure.azure, args=args)

            with subtests.test(msg=f"Testing without {env_var} being set"):
                assert actual.exit_code == expected_exit_code
                assert actual.output == expected_output


@patch("azure.common.credentials.ServicePrincipalCredentials")
@patch("azure.mgmt.resource.ResourceManagementClient")
def test_azure_with_one_missing_env_var_and_related_argument(cred, client, subtests):
    """
    This test case verifies that the azure provider will run properly if all
    but one environment variable is set and the missing value is provided
    as command line argument.

    In order to avoid calls to the Microsoft Azure API, both external
    dependencies are patched as mock objects.
    """
    for env_var in ENV_VARS:
        subset = {env.env: env.value for env in ENV_VARS if env != env_var}

        with patch.dict(os.environ, subset):
            expected_exit_code = 0
            expected_output = f"Found no deletable resource groups for prefix test.\n"

            args = ["--prefix=test", f"{env_var.arg}={env_var.value}", "--yes"]

            runner = click.testing.CliRunner()
            actual = runner.invoke(azure.azure, args=args)

            with subtests.test(msg=f"Testing with {env_var} being set as argument"):
                assert actual.exit_code == expected_exit_code
                assert actual.output == expected_output



"""
agipy enables you to destroy cloud infrastructure in a hurry.
Copyright (C) 2019  Nils Müller<shimst3r@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>
"""

# Most of the test cases in this test suite are about testing to import
# submodules that will not be used.
# pylint: disable=import-outside-toplevel, unused-import


def test_import_agipy_module():
    """
    test_import_agipy_module makes sure the agipy module can be imported
    properly.
    """
    try:
        import agipy
    except ImportError:
        assert False
    else:
        assert True


def test_import_azure_provider():
    """
    test_import_azure_provider makes sure the azure provider module can be
    imported properly.
    """
    try:
        from agipy.providers import azure
    except ImportError:
        assert False
    else:
        assert True



#!/usr/bin/env python3

"""
agipy enables you to destroy cloud infrastructure in a hurry.
Copyright (C) 2019  Nils Müller <shimst3r@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>
"""


from setuptools import setup

import m2r

__version__ = "0.2.2"

ENTRY_POINTS = {"console_scripts": ["agipy = agipy.agipy:interface"]}

# The README is written using markdown, PyPI expects the long description
# to be written in reStructuredText. Thus the `m2r` converter is used
# for converting between the two.
with open("README.md", "r", encoding="utf-8") as f_in:
    LONG_DESCRIPTION = m2r.convert(f_in.read())

PACKAGES = ["agipy", "agipy.providers"]

REQUIRES = {
    "azure": ["azure-common", "azure-mgmt-resource", "msrestazure"],
    "general": ["Click"],
}

setup(
    author="Nils P. Müller",
    author_email="shimst3r@gmail.com",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="destroy cloud infrastructure in a hurry",
    download_url=f"https://github.com/shimst3r/agipy/archive/{__version__}.tar.gz",
    entry_points=ENTRY_POINTS,
    install_requires=[package for section in REQUIRES.values() for package in section],
    keywords=["azure", "cli", "cloud", "devops", "sysadmin", "tool"],
    license="GPL",
    long_description=LONG_DESCRIPTION,
    name="agipy",
    packages=PACKAGES,
    python_requires=">=3.7",
    url="https://github.com/shimst3r/agipy",
    version=__version__,
)



"""
agipy enables you to destroy cloud infrastructure in a hurry.
Copyright (C) 2019  Nils Müller<shimst3r@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>
"""
import click

from .providers import azure

# pylint and Click do not work well together in all cases, see:
# https://stackoverflow.com/questions/49680191/click-and-pylint
# pylint: disable=no-value-for-parameter
@click.group()
def interface():
    """
    agipy enables you to destroy cloud infrastructure in a hurry
    """


interface.add_command(azure.azure)


if __name__ == "__main__":
    interface()



"""
agipy enables you to destroy cloud infrastructure in a hurry.
Copyright (C) 2019  Nils Müller<shimst3r@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>
"""
import concurrent.futures
import os
import sys

import click
from azure.common import credentials  # pylint: disable=no-name-in-module
from azure.mgmt import resource  # pylint: disable=no-name-in-module
from msrestazure.azure_exceptions import CloudError


@click.command()
@click.option("--prefix", required=True, help="Prefix of resource groups to delete.")
@click.option("--client-id", required=False, help="Your Service Principal ID")
@click.option("--client-secret", required=False, help="Your Service Principal Secret")
@click.option("--subscription-id", required=False, help="Your Subscription ID")
@click.option("--tenant-id", required=False, help="Your Tenant ID")
@click.confirmation_option(
    prompt="Are you sure you want to delete all resource groups?"
)
def azure(prefix, client_id, client_secret, subscription_id, tenant_id):
    """
    azure is a command that lets you delete resource groups on
    Microsoft Azure's public cloud infrastructure.
    """
    # The AzureProvider is initialised using a combination of environment
    # variables and command line arguments.
    # If either combination is not set properly, the process is exited.
    try:
        client_id = client_id or os.environ["AZURE_CLIENT_ID"]
        client_secret = client_secret or os.environ["AZURE_CLIENT_SECRET"]
        subscription_id = subscription_id or os.environ["AZURE_SUBSCRIPTION_ID"]
        tenant_id = tenant_id or os.environ["AZURE_TENANT_ID"]
    except KeyError as error:
        click.echo(f"azure provider is missing the {error}.")
        sys.exit(1)
    else:
        provider = AzureProvider(
            client_id=client_id,
            client_secret=client_secret,
            subscription_id=subscription_id,
            tenant_id=tenant_id,
        )

    provider.delete(prefix=prefix)


class AzureProvider:
    """
    AzureProvider implements functionality to delete resource groups on
    Microsoft Azure's public cloud infrastructure.
    """

    def __init__(self, client_id, client_secret, subscription_id, tenant_id):
        _credentials = credentials.ServicePrincipalCredentials(
            client_id=client_id, secret=client_secret, tenant=tenant_id
        )
        self.client = resource.ResourceManagementClient(
            credentials=_credentials, subscription_id=subscription_id
        )

    @property
    def resource_groups(self):
        """
        resource_groups returns a list of resource groups for the given client.
        """
        return list(self.client.resource_groups.list())

    def delete(self, prefix: str):
        """
        delete connects to the Azure public cloud and deletes all resources
        in resource groups specified py prefix.
        """

        self._delete_resource_groups_by_prefix(prefix=prefix)

    def _delete_resource_group_by_name(self, rg_name: str):
        """
        _delete_resource_group_by_name deletes a resource group based on
        its name.
        """
        awaitable = self.client.resource_groups.delete(rg_name)

        try:
            awaitable.wait()
        except CloudError as error:
            click.echo(error.message)
        else:
            click.echo(f"Succesfully deleted resource group {rg_name}.")

    def _delete_resource_groups_by_prefix(self, prefix: str):
        """
        _delete_resource_groups_by_prefix deletes all resource groups for a
        given subscription based on a given prefix.
        """
        deletables = [
            rgx for rgx in self.resource_groups if rgx.name.startswith(prefix)
        ]

        if not deletables:
            click.echo(f"Found no deletable resource groups for prefix {prefix}.")
            return

        with concurrent.futures.ThreadPoolExecutor() as executor:
            for rgx in deletables:
                click.echo(
                    f"Found deletable resource group {rgx.name}. Initialise deletion."
                )
                executor.submit(self._delete_resource_group_by_name, rgx.name)

        click.echo(
            f"Succesfully deleted {len(deletables)} resource groups for prefix {prefix}."
        )
